<?xml version="1.0" encoding="UTF-8"?>

<!--
 @file docgen.toc.xsl
 
 This stylesheet is resonsible for taking a message definition
 file (mxsd file) and producing data for producing a JavaScript
 tree for navigating the mxsd documentation.
 This transform is invoked by the HtmlDocWriteHelper class.
-->

<!DOCTYPE xsl:stylesheet [
<!ENTITY eol     "&#10;">
<!ENTITY space   "&#32;">
<!ENTITY nbsp    "&#160;">
]>

<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  xmlns:xalan="http://xml.apache.org/xslt">

  <xsl:param name="title">Page Title</xsl:param>
  <xsl:param name="filename">file.mxsd</xsl:param>
  <xsl:param name="relativePath"></xsl:param>
  <xsl:param name="outputPath"></xsl:param>
  <xsl:param name="nlsfile">docgen_nls.xml</xsl:param>

  <xsl:variable name="mainframe">
    <xsl:value-of select="concat($filename,'.main.html')"/>
  </xsl:variable>

	<xsl:output method="text" version="1.0" encoding="UTF-8" media-type="text/plain" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <xsl:variable name="pathsep" select="'.'" />

  <!--
  	Namespace processing - se the main stylesheet for details.
  	This is needed in the TOC for following types.
  	-->
  <xsl:variable name="namespaceTarget">
  	<xsl:choose>
  		<xsl:when test="/xsd:schema/@targetNamespace">
  			<xsl:value-of select="/xsd:schema/@targetNamespace"/>
  		</xsl:when>
  	</xsl:choose>
  </xsl:variable>
	
  <xsl:variable name="namespacePrefix">
  	<xsl:choose>
  		<xsl:when test="/xsd:schema/@targetNamespace">
			<xsl:for-each select="/xsd:schema/namespace::*">
				<xsl:if test="self::node() = $namespaceTarget">
		  			<xsl:value-of select="local-name()"/>
				</xsl:if>
			</xsl:for-each>
  		</xsl:when>
  	</xsl:choose>
  </xsl:variable>

<!--
   ##    #####        #  #    #   ####    #####  #    #    ##    #    #  ######
  #  #   #    #       #  #    #  #          #    ##   #   #  #   ##  ##  #
 #    #  #    #       #  #    #   ####      #    # #  #  #    #  # ## #  #####
 ######  #    #       #  #    #       #     #    #  # #  ######  #    #  #
 #    #  #    #  #    #  #    #  #    #     #    #   ##  #    #  #    #  #
 #    #  #####    ####    ####    ####      #    #    #  #    #  #    #  ######

		This template adjusts the name of a entity, allowing for namespaces.
		When the schema has a namespace prefix of "a", and the entity is
		"a:complexType1", "complexType1" is returned.
		For all other combinations, "a:complexType1" is returned.
-->
	<xsl:template name="adjustnameofentity">
		<xsl:param name="entityname"/>
		
		<!--
			Just have this line if you want to disable adjustment
			for debugging
		<xsl:value-of select="$entityname"/>
      	-->

      	<xsl:variable name="namespaceForEntity">
  			<xsl:if test="contains($entityname,':')">
  				<xsl:value-of select="substring-before($entityname,':')"/>
  			</xsl:if>
      	</xsl:variable>
      	
    	<xsl:choose>
      		<xsl:when test="string-length($namespaceForEntity)>0 and $namespaceForEntity=$namespacePrefix">
	  			<xsl:value-of select="substring-after($entityname,':')"/>
   			</xsl:when>
   			<xsl:otherwise>
	  			<xsl:value-of select="$entityname"/>
   			</xsl:otherwise>
      	</xsl:choose>
      	
	</xsl:template>
	
<!--
      #
     #
    #
   #
  #
 #
#

	This is where the processing starts.
	The aim is to mimic the form of the "Outline" window in the
	eclipse tooling.
-->
  <xsl:template match="/">
    <xsl:variable name="prefix" select="$mainframe"/>
    <xsl:variable name="depth" select="'0'"/>

    <xsl:call-template name="outputPrologue">
      <xsl:with-param name="prefix" select="$prefix"/>
      <xsl:with-param name="depth" select="$depth"/>
    </xsl:call-template>
    <xsl:call-template name="messages">
      <xsl:with-param name="prefix" select="$prefix"/>
      <xsl:with-param name="depth" select="$depth"/>
    </xsl:call-template>
    <xsl:call-template name="types">
      <xsl:with-param name="prefix" select="$prefix"/>
      <xsl:with-param name="depth" select="$depth"/>
    </xsl:call-template>
    <xsl:call-template name="groups">
      <xsl:with-param name="prefix" select="$prefix"/>
      <xsl:with-param name="depth" select="$depth"/>
    </xsl:call-template>
    <xsl:call-template name="elementsAndAttributes">
      <xsl:with-param name="prefix" select="$prefix"/>
      <xsl:with-param name="depth" select="$depth"/>
    </xsl:call-template>
    <!--
    Output a footer showing date generated, etc.
    -->
  </xsl:template>

<!--
  ####   #    #   #####  #####   #    #   #####  #####   #####    ####   #
 #    #  #    #     #    #    #  #    #     #    #    #  #    #  #    #  #
 #    #  #    #     #    #    #  #    #     #    #    #  #    #  #    #  #
 #    #  #    #     #    #####   #    #     #    #####   #####   #    #  #
 #    #  #    #     #    #       #    #     #    #       #   #   #    #  #
  ####    ####      #    #        ####      #    #       #    #   ####   ######
-->
  <xsl:template name="outputPrologue">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>

    <xsl:variable name="myself" select="'foldersTree'"/>
    <xsl:variable name="mychild" select="concat('aux',$depth)"/>

	<!-- Trace to confirm the NLS file -->
	<xsl:call-template name="blockcomment">
		<xsl:with-param name="text">
			<xsl:text>NLS File:</xsl:text>
			<xsl:value-of select="$nlsfile" />
		</xsl:with-param>
	</xsl:call-template>
	
	<xsl:text>var HIGHLIGHT = 1&eol;</xsl:text>
	<xsl:text>var HIGHLIGHT_BG = '#316ac5'&eol;</xsl:text>
	<xsl:text>var ICONPATH = "</xsl:text>
	<xsl:value-of select="$relativePath"/>
	<xsl:text>images/"&eol;</xsl:text>
	
    <xsl:value-of select="$myself"/>
    <xsl:text>= gFld(&quot;</xsl:text>
    <xsl:value-of select="$messagesetString"/>
    <xsl:text>", "</xsl:text>
    <xsl:value-of select="concat($relativePath,'index.html')"/>
    <xsl:text>", "MSC")&eol;</xsl:text>

    <xsl:call-template name="writeTreeEntry">
    	<xsl:with-param name="child" select="$mychild" />
    	<xsl:with-param name="parent" select="$myself" />
    	<xsl:with-param name="title" select="$filename"/>
    	<xsl:with-param name="target" select="$prefix" />
    	<xsl:with-param name="iconname" select="'MDC'" />
    </xsl:call-template>

  </xsl:template>

<!--
 #    #  ######   ####    ####     ##     ####   ######   ####
 ##  ##  #       #       #        #  #   #    #  #       #
 # ## #  #####    ####    ####   #    #  #       #####    ####
 #    #  #            #       #  ######  #  ###  #            #
 #    #  #       #    #  #    #  #    #  #    #  #       #    #
 #    #  ######   ####    ####   #    #   ####   ######   ####
-->
  <xsl:template name="messages">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />

  	<xsl:call-template name="blockcomment">
  		<xsl:with-param name="text">Messages</xsl:with-param>
  	</xsl:call-template>

  	<xsl:variable name="myparent" select="concat('aux',$depth)" />
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />
  	<xsl:variable name="fullpath" select="concat($prefix,'#messages/')" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$myself" />
  		<xsl:with-param name="parent" select="$myparent" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="'MC'" />
  		<xsl:with-param name="title">
  			<xsl:call-template name="lookupText">
  				<xsl:with-param name="key" select="'messages'" />
  			</xsl:call-template>
  		</xsl:with-param>
  	</xsl:call-template>

  	<xsl:for-each select="/xsd:schema/xsd:element//*[local-name()='MRMessage']">
  		<xsl:apply-templates select=".">
  			<xsl:with-param name="prefix" select="concat($fullpath,'message/')" />
  			<xsl:with-param name="depth" select="$depth" />
  		</xsl:apply-templates>
  	</xsl:for-each>

  </xsl:template>

  <!--
 #####   #   #  #####   ######   ####
   #      # #   #    #  #       #
   #       #    #    #  #####    ####
   #       #    #####   #            #
   #       #    #       #       #    #
   #       #    #       ######   ####
-->
  <xsl:template name="types">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>

	<xsl:call-template name="blockcomment">
	  <xsl:with-param name="text">Types</xsl:with-param>
	</xsl:call-template>

    <xsl:variable name="myparent" select="concat('aux',$depth)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>
    <xsl:variable name="fullpath" select="concat($prefix,'#types/')"/>

    <xsl:call-template name="writeTreeEntry">
    	<xsl:with-param name="child" select="$myself" />
    	<xsl:with-param name="parent" select="$myparent" />
    	<xsl:with-param name="target" select="$fullpath" />
    	<xsl:with-param name="iconname" select="'TC'" />
    	<xsl:with-param name="title">
    		<xsl:call-template name="lookupText">
    			<xsl:with-param name="key" select="'types'" />
    		</xsl:call-template>
    	</xsl:with-param>
    </xsl:call-template>

    <xsl:for-each select="/xsd:schema/xsd:complexType | /xsd:schema/xsd:simpleType">
      <xsl:apply-templates select=".">
        <xsl:with-param name="prefix" select="concat($fullpath,'type/')"/>
        <xsl:with-param name="depth" select="$depth"/>
      </xsl:apply-templates>
    </xsl:for-each>
  </xsl:template>

<!--
  ####   #####    ####   #    #  #####    ####
 #    #  #    #  #    #  #    #  #    #  #
 #       #    #  #    #  #    #  #    #   ####
 #  ###  #####   #    #  #    #  #####        #
 #    #  #   #   #    #  #    #  #       #    #
  ####   #    #   ####    ####   #        ####
-->
  <xsl:template name="groups">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>

	<xsl:call-template name="blockcomment">
	  <xsl:with-param name="text">Groups</xsl:with-param>
	</xsl:call-template>
   	
    <xsl:variable name="myparent" select="concat('aux',$depth)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>
    <xsl:variable name="fullpath" select="concat($prefix,'#groups/')"/>

    <xsl:call-template name="writeTreeEntry">
    	<xsl:with-param name="child" select="$myself" />
    	<xsl:with-param name="parent" select="$myparent" />
    	<xsl:with-param name="target" select="$fullpath" />
    	<xsl:with-param name="iconname" select="'GC'" />
    	<xsl:with-param name="title">
    		<xsl:call-template name="lookupText">
    			<xsl:with-param name="key" select="'groups'" />
    		</xsl:call-template>
    	</xsl:with-param>
    </xsl:call-template>

    <!-- Process any groups -->
    <xsl:for-each select="/xsd:schema/xsd:group | /xsd:schema/xsd:attributeGroup">
      <xsl:apply-templates select=".">
        <xsl:with-param name="prefix" select="concat($fullpath,local-name(),'/')"/>
        <xsl:with-param name="depth" select="$depth"/>
      </xsl:apply-templates>
    </xsl:for-each>

  </xsl:template>

<!--
 #    #  #####   #    #  ######   ####    ####     ##     ####   ######  
 ##  ##  #    #  ##  ##  #       #       #        #  #   #    #  #       
 # ## #  #    #  # ## #  #####    ####    ####   #    #  #       #####   
 #    #  #####   #    #  #            #       #  ######  #  ###  #       
 #    #  #   #   #    #  #       #    #  #    #  #    #  #    #  #       
 #    #  #    #  #    #  ######   ####    ####   #    #   ####   ######  
-->
  <xsl:template match="*[local-name()='MRMessage']">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />

  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />
  	<xsl:variable name="atname" select="ancestor::xsd:element/@name" />
  	<xsl:variable name="icontype" select="'MO'" />
  	<xsl:variable name="fullpath" select="concat($prefix,$atname,$pathsep)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="$atname" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:for-each select="ancestor::xsd:element">
  		<xsl:call-template name="processElement">
  			<xsl:with-param name="prefix" select="concat($mainframe,'#elementsAndAttributes/element/',$atname,$pathsep)" />
  			<xsl:with-param name="depth" select="$depth" />
  		</xsl:call-template>
  	</xsl:for-each>

  </xsl:template>

  <!--
 ######  #        ####     #       ##     #####   ####
 #       #       #         #      #  #      #    #
 #####   #        ####   #####   #    #     #     ####
 #       #            #    #     ######     #         #
 #       #       #    #    #     #    #     #    #    #
 ######  ######   ####           #    #     #     ####
-->
  <xsl:template name="elementsAndAttributes">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>

	<xsl:call-template name="blockcomment">
	  <xsl:with-param name="text">Elements and Attributes</xsl:with-param>
	</xsl:call-template>

    <xsl:variable name="myparent" select="concat('aux',$depth)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>
    <xsl:variable name="fullpath" select="concat($prefix,'#elementsAndAttributes/')"/>

    <xsl:call-template name="writeTreeEntry">
    	<xsl:with-param name="child" select="$myself" />
    	<xsl:with-param name="parent" select="$myparent" />
    	<xsl:with-param name="target" select="$fullpath" />
    	<xsl:with-param name="iconname" select="'EAC'" />
    	<xsl:with-param name="title">
    		<xsl:call-template name="lookupText">
    			<xsl:with-param name="key" select="'elementsAndAttributes'" />
    		</xsl:call-template>
    	</xsl:with-param>
    </xsl:call-template>

    <xsl:for-each select="/xsd:schema/xsd:element | /xsd:schema/xsd:attribute">
      <xsl:apply-templates select=".">
        <xsl:with-param name="prefix" select="concat($fullpath,local-name(),'/')"/>
        <xsl:with-param name="depth" select="$depth"/>
      </xsl:apply-templates>
    </xsl:for-each>
  </xsl:template>

<!--
 ######  #       ######  #    #  ######  #    #   #####
 #       #       #       ##  ##  #       ##   #     #
 #####   #       #####   # ## #  #####   # #  #     #
 #       #       #       #    #  #       #  # #     #
 #       #       #       #    #  #       #   ##     #
 ######  ######  ######  #    #  ######  #    #     #
-->
  <xsl:template match="xsd:element[@name]">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>

    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

	<!-- Detect compound elements here because of the special icon -->
	<xsl:variable name="icontype">
		<xsl:choose>
			<!-- 
				Compound elements have specific rules.
				Modify this line as the rules alter
			-->
			<xsl:when test="xsd:complexType/@mixed='true' and xsd:complexType/xsd:sequence/xsd:element/@maxOccurs='0' and xsd:complexType/xsd:sequence/xsd:element/@minOccurs='0' and starts-with(xsd:complexType/xsd:sequence/xsd:element/@type,'ComIbmMrm_BaseValue') and starts-with(xsd:complexType/xsd:sequence/xsd:element/@name,'ComIbmMrm_21')">
				<xsl:value-of select="'SBT'"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'ED'"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>
    
	<xsl:variable name="isRecursive" select="contains($allancestors,$uniqueid)"/>
    <xsl:variable name="title">
    	<xsl:choose>
    		<xsl:when test="$isRecursive">
    			<xsl:value-of select="concat(@name,$recursionIndicatorString)" />
    		</xsl:when>
    		<xsl:otherwise>
    			<xsl:value-of select="@name" />
    		</xsl:otherwise>
    	</xsl:choose>
    </xsl:variable>

    <xsl:call-template name="writeTreeEntry">
    	<xsl:with-param name="child" select="$mychild" />
    	<xsl:with-param name="parent" select="$myself" />
    	<xsl:with-param name="title" select="$title"/>
    	<xsl:with-param name="target" select="$fullpath" />
    	<xsl:with-param name="iconname" select="$icontype" />
    </xsl:call-template>

    <xsl:if test="not($isRecursive)">
    	<xsl:call-template name="processElement">
    		<xsl:with-param name="prefix" select="$fullpath" />
    		<xsl:with-param name="depth" select="$depth" />
    		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
    	</xsl:call-template>
    </xsl:if>

  </xsl:template>

<!--
 ######  #       ######  #    #  ######  #    #   #####
 #       #       #       ##  ##  #       ##   #     #
 #####   #       #####   # ## #  #####   # #  #     #
 #       #       #       #    #  #       #  # #     #
 #       #       #       #    #  #       #   ##     #
 ######  ######  ######  #    #  ######  #    #     #
 
With references, we keep expanding tree, but link to the true definition
This needs to be namespace aware

-->
  <xsl:template match="xsd:element[@ref]">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>

<!--
	<xsl:call-template name="blockcomment">
		<xsl:with-param name="text">
			<xsl:text>ELEMENT REF</xsl:text>
		</xsl:with-param>
	</xsl:call-template>
-->	
  	<xsl:variable name="uniqueid">
    	<xsl:call-template name="uniqueref" />
  	</xsl:variable>

	<xsl:variable name="icontype" select="'EU'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>
	<xsl:variable name="isRecursive" select="contains($allancestors,$uniqueid)"/>

    <xsl:variable name="title">
    	<xsl:choose>
    		<xsl:when test="$isRecursive">
    			<xsl:value-of select="concat(@ref,$recursionIndicatorString)" />
    		</xsl:when>
    		<xsl:otherwise>
    			<xsl:value-of select="@ref" />
    		</xsl:otherwise>
    	</xsl:choose>
    </xsl:variable>

    <xsl:call-template name="writeTreeEntry">
    	<xsl:with-param name="child" select="$mychild" />
    	<xsl:with-param name="parent" select="$myself" />
    	<xsl:with-param name="title" select="$title"/>
    	<xsl:with-param name="target" select="$fullpath" />
    	<xsl:with-param name="iconname" select="$icontype" />
    </xsl:call-template>

    <xsl:if test="not($isRecursive)">
	    <xsl:variable name="type">
    		<xsl:call-template name="adjustnameofentity">
    			<xsl:with-param name="entityname" select="@ref" />
	    	</xsl:call-template>
    	</xsl:variable>
    	<xsl:variable name="newfullpath" select="concat($mainframe,'#elementsAndAttributes/element/',$type,$pathsep)" />
    	<xsl:for-each select="/xsd:schema/xsd:element[@name=$type]">
    		<xsl:call-template name="processElement">
    			<xsl:with-param name="prefix" select="$newfullpath" />
    			<xsl:with-param name="depth" select="$depth" />
    			<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
    		</xsl:call-template>
    	</xsl:for-each>
    </xsl:if>
  </xsl:template>

<!--
 #####   #####    ####    ####   ######   ####    ####   ######  #       ###### 
 #    #  #    #  #    #  #    #  #       #       #       #       #       #      
 #    #  #    #  #    #  #       #####    ####    ####   #####   #       #####  
 #####   #####   #    #  #       #            #       #  #       #       #      
 #       #   #   #    #  #    #  #       #    #  #    #  #       #       #      
 #       #    #   ####    ####   ######   ####    ####   ######  ######  ###### 
-->

  <xsl:template name="processElement">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>

    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>
   	<xsl:variable name="mygrandchild" select="concat('aux',$depth+3)" />

    <xsl:choose>

      <xsl:when test="xsd:complexType">
      	<xsl:choose>
			<!-- 
				Compound elements have specific rules.
				Modify this line as the rules alter
			-->
			<xsl:when test="xsd:complexType/@mixed='true' and xsd:complexType/xsd:sequence/xsd:element/@maxOccurs='0' and xsd:complexType/xsd:sequence/xsd:element/@minOccurs='0' and starts-with(xsd:complexType/xsd:sequence/xsd:element/@type,'ComIbmMrm_BaseValue') and starts-with(xsd:complexType/xsd:sequence/xsd:element/@name,'ComIbmMrm_21')">
				<!--
				<xsl:call-template name="blockcomment">
					<xsl:with-param name="text">
						<xsl:text>Compound Element</xsl:text>
					</xsl:with-param>
				</xsl:call-template>
				-->
				<!--
					Now select the group reference and follow that
				-->
		      	<xsl:apply-templates select="xsd:complexType/xsd:sequence/xsd:group[@ref]" mode="notreeentry">
		      		<xsl:with-param name="prefix" select="$prefix" />
      				<xsl:with-param name="depth" select="$depth" />
		      		<xsl:with-param name="allancestors" select="$allancestors" />
		      	</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
		      	<xsl:apply-templates select="xsd:complexType" mode="localcomplextype">
      				<xsl:with-param name="prefix" select="$prefix" />
		      		<xsl:with-param name="depth" select="$depth+1" />
		      		<xsl:with-param name="allancestors" select="$allancestors" />
		      	</xsl:apply-templates>
			</xsl:otherwise>
   		</xsl:choose>
      </xsl:when>

      <xsl:when test="xsd:simpleType">
      	<xsl:apply-templates select="xsd:simpleType" mode="localsimpletype">
      		<xsl:with-param name="prefix" select="$prefix" />
      		<xsl:with-param name="depth" select="$depth+1" />
      		<xsl:with-param name="allancestors" select="$allancestors" />
      	</xsl:apply-templates>
      </xsl:when>

      <xsl:when test="@type">
      	<!-- Need to handle namespaces -->
      	<xsl:variable name="type">
      		<xsl:call-template name="adjustnameofentity">
      			<xsl:with-param name="entityname" select="@type" />
      		</xsl:call-template>
      	</xsl:variable>

 		<xsl:variable name="uniqueid" select="$type"/>
      	<xsl:variable name="fullpath" select="concat($mainframe,'#types/type/',$uniqueid,$pathsep)" />

      	<xsl:apply-templates select="/xsd:schema/xsd:complexType[@name=$type]" mode="notreeentry">
      		<xsl:with-param name="prefix" select="$fullpath" />
      		<xsl:with-param name="depth" select="$depth" />
      		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
      	</xsl:apply-templates>

        <xsl:apply-templates select="/xsd:schema/xsd:simpleType[@name=$type]" mode="notreeentry">
            <xsl:with-param name="prefix" select="$fullpath" />
            <xsl:with-param name="depth" select="$depth+1" />
            <xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
        </xsl:apply-templates>

      </xsl:when>
      <xsl:otherwise>
        <xsl:text>// Element has no type!</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

<!--
   ##     #####   #####  #####      #    #####   #    #   #####  ######
  #  #      #       #    #    #     #    #    #  #    #     #    #
 #    #     #       #    #    #     #    #####   #    #     #    #####
 ######     #       #    #####      #    #    #  #    #     #    #
 #    #     #       #    #   #      #    #    #  #    #     #    #
 #    #     #       #    #    #     #    #####    ####      #    ######
-->
  <xsl:template match="xsd:attribute[@name]">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquename" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'AD'" />
  	<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="@name" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:call-template name="processAttribute">
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth" />
  		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:call-template>

  </xsl:template>

  <!--
   ##     #####   #####  #####      #    #####   #    #   #####  ######
  #  #      #       #    #    #     #    #    #  #    #     #    #
 #    #     #       #    #    #     #    #####   #    #     #    #####
 ######     #       #    #####      #    #    #  #    #     #    #
 #    #     #       #    #   #      #    #    #  #    #     #    #
 #    #     #       #    #    #     #    #####    ####      #    ######
-->
  <xsl:template match="xsd:attribute[@ref]">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>

<!--
    <xsl:variable name="uniqueref">
    	<xsl:call-template name="uniqueref" />
    </xsl:variable>
-->
  	<xsl:variable name="uniqueid">
    	<xsl:call-template name="uniqueref" />
  	</xsl:variable>

	<xsl:variable name="icontype" select="'AU'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>
	<xsl:variable name="isRecursive" select="contains($allancestors,$uniqueid)"/>

    <xsl:variable name="title">
    	<xsl:choose>
    		<xsl:when test="$isRecursive">
    			<xsl:value-of select="concat(@ref,$recursionIndicatorString)" />
    		</xsl:when>
    		<xsl:otherwise>
    			<xsl:value-of select="@ref" />
    		</xsl:otherwise>
    	</xsl:choose>
    </xsl:variable>

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="$title" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:if test="not($isRecursive)">
	    <xsl:variable name="type">
    		<xsl:call-template name="adjustnameofentity">
    			<xsl:with-param name="entityname" select="@ref" />
	    	</xsl:call-template>
    	</xsl:variable>
  		<xsl:variable name="newfullpath" select="concat($mainframe,'#elementsAndAttributes/attribute/',@ref,$pathsep)" />
  		<xsl:for-each select="/xsd:schema/xsd:attribute[@name=$type]">
  			<xsl:call-template name="processAttribute">
  				<xsl:with-param name="prefix" select="$newfullpath" />
  				<xsl:with-param name="depth" select="$depth" />
		    	<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  			</xsl:call-template>
  		</xsl:for-each>
  	</xsl:if>

  </xsl:template>

<!--
 #####   #####    ####    ####   ######   ####    ####     ##     #####   #####
 #    #  #    #  #    #  #    #  #       #       #        #  #      #       #  
 #    #  #    #  #    #  #       #####    ####    ####   #    #     #       #  
 #####   #####   #    #  #       #            #       #  ######     #       #  
 #       #   #   #    #  #    #  #       #    #  #    #  #    #     #       #  
 #       #    #   ####    ####   ######   ####    ####   #    #     #       #  
-->
  <xsl:template name="processAttribute">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:variable name="fullpath" select="$prefix" />
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />
  	<xsl:choose>
  		<!-- element has local complex type -->
  		<xsl:when test="xsd:complexType">
  			<xsl:apply-templates>
  				<xsl:with-param name="prefix" select="$fullpath" />
  				<xsl:with-param name="depth" select="$depth + 1" />
  				<xsl:with-param name="allancestors" select="$allancestors" />
  			</xsl:apply-templates>
  		</xsl:when>
  		<!-- element has local simple type -->
  		<xsl:when test="xsd:simpleType">
  			<xsl:apply-templates mode="localsimpletype">
  				<xsl:with-param name="prefix" select="$fullpath" />
  				<xsl:with-param name="depth" select="$depth + 1" />
  				<xsl:with-param name="allancestors" select="$allancestors" />
  			</xsl:apply-templates>
  		</xsl:when>
  		<!-- element type declared globally -->
  		<xsl:when test="@type">
  			<xsl:variable name="type" select="@type" />
  			<xsl:apply-templates select="/xsd:schema/xsd:complexType[@name=$type]">
  				<xsl:with-param name="prefix" select="$fullpath" />
  				<xsl:with-param name="depth" select="$depth + 1" />
  				<xsl:with-param name="allancestors" select="$allancestors" />
  			</xsl:apply-templates>
  		</xsl:when>
  		<xsl:otherwise>
  			<xsl:text>// Attribute has no type!</xsl:text>
  		</xsl:otherwise>
  	</xsl:choose>

  </xsl:template>
  <!--
   ##    #    #   #   #
  #  #   ##   #    # #
 #    #  # #  #     #
 ######  #  # #     #
 #    #  #   ##     #
 #    #  #    #     #
-->
  <xsl:template match="xsd:any">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquewildcard" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'WE'" />
  	<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  		<xsl:with-param name="title" select="$wildcardElementString"/>
  	</xsl:call-template>

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth + 1" />
  		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:apply-templates>
  </xsl:template>

  <!--
   ##    #    #   #   #    ##     #####   #####  #####      #    #####   #    # 
  #  #   ##   #    # #    #  #      #       #    #    #     #    #    #  #    # 
 #    #  # #  #     #    #    #     #       #    #    #     #    #####   #    # 
 ######  #  # #     #    ######     #       #    #####      #    #    #  #    # 
 #    #  #   ##     #    #    #     #       #    #   #      #    #    #  #    # 
 #    #  #    #     #    #    #     #       #    #    #     #    #####    ####  
-->
  <xsl:template match="xsd:anyAttribute">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquewildcard" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'WA'" />
  	<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  		<xsl:with-param name="title" select="$wildcardAttributeString" />
  	</xsl:call-template>

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth + 1" />
  		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:apply-templates>
  </xsl:template>

  <!--
  ####      #    #    #  #####   #       ######   #####   #   #  #####   ###### 
 #          #    ##  ##  #    #  #       #          #      # #   #    #  #      
  ####      #    # ## #  #    #  #       #####      #       #    #    #  #####  
      #     #    #    #  #####   #       #          #       #    #####   #      
 #    #     #    #    #  #       #       #          #       #    #       #      
  ####      #    #    #  #       ######  ######     #       #    #       ###### 
-->
  <xsl:template match="xsd:simpleType[@name]">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquename"/>
  	</xsl:variable>

 	<xsl:variable name="icontype">
  		<xsl:choose>
  			<xsl:when test="xsd:restriction">
  				<xsl:value-of select="'STDA'" />
  			</xsl:when>
  			<xsl:when test="xsd:list">
  				<xsl:value-of select="'STDL'" />
  			</xsl:when>
  			<xsl:when test="xsd:union">
  				<xsl:value-of select="'STDU'" />
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:value-of select="'NONE'" />
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:variable>
   	
  	<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="@name" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:call-template name="processSimpleType">
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth + 1" />
  		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:call-template>
  </xsl:template>

  <!--
  ####      #    #    #  #####   #       ######   #####   #   #  #####   ###### 
 #          #    ##  ##  #    #  #       #          #      # #   #    #  #      
  ####      #    # ## #  #    #  #       #####      #       #    #    #  #####  
      #     #    #    #  #####   #       #          #       #    #####   #      
 #    #     #    #    #  #       #       #          #       #    #       #      
  ####      #    #    #  #       ######  ######     #       #    #       ###### 
-->
  <xsl:template match="xsd:simpleType[@name]" mode="notreeentry">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:call-template name="processSimpleType">
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="depth" select="$depth" />
  		<xsl:with-param name="allancestors" select="$allancestors" />
  	</xsl:call-template>
  </xsl:template>

  <!--
  ####      #    #    #  #####   #       ######   #####   #   #  #####   ###### 
 #          #    ##  ##  #    #  #       #          #      # #   #    #  #      
  ####      #    # ## #  #    #  #       #####      #       #    #    #  #####  
      #     #    #    #  #####   #       #          #       #    #####   #      
 #    #     #    #    #  #       #       #          #       #    #       #      
  ####      #    #    #  #       ######  ######     #       #    #       ###### 
-->
  <xsl:template match="xsd:simpleType" mode="localsimpletype">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />
  	
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquename" />
  	</xsl:variable>

    <xsl:variable name="icontype">
  		<xsl:choose>
  			<xsl:when test="xsd:restriction">
  				<xsl:value-of select="'STDA'" />
  			</xsl:when>
  			<xsl:when test="xsd:list">
  				<xsl:value-of select="'STDL'" />
  			</xsl:when>
  			<xsl:when test="xsd:union">
  				<xsl:value-of select="'STDU'" />
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:value-of select="'NONE'" />
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:variable>

 	<xsl:variable name="base">
  		<xsl:choose>
  			<xsl:when test="xsd:restriction">
  				<xsl:value-of select="xsd:restriction/@base" />
  			</xsl:when>
  			<xsl:when test="xsd:list">
  				<xsl:value-of select="xsd:list/@itemType" />
  			</xsl:when>
  			<xsl:when test="xsd:union">
                <xsl:call-template name="lookupText">
                    <xsl:with-param name="key" select="'localsimpletype'" />
                </xsl:call-template>
  			</xsl:when>
  			<xsl:otherwise>
  				<xsl:value-of select="'NONE'" />
  			</xsl:otherwise>
  		</xsl:choose>
  	</xsl:variable>

  	<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="concat($base, ' -')" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:call-template name="processSimpleType">
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth+1" />
  		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:call-template>
  </xsl:template>

  <!--
  ####      #    #    #  #####   #       ######   #####   #   #  #####   ###### 
 #          #    ##  ##  #    #  #       #          #      # #   #    #  #      
  ####      #    # ## #  #    #  #       #####      #       #    #    #  #####  
      #     #    #    #  #####   #       #          #       #    #####   #      
 #    #     #    #    #  #       #       #          #       #    #       #      
  ####      #    #    #  #       ######  ######     #       #    #       ###### 
-->
  <xsl:template match="xsd:simpleType" mode="localsimpletypeunion">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />
  	
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquename" />
  	</xsl:variable>

 	<xsl:variable name="icontype" select="'STDU'" />

	<xsl:variable name="mytitle">
   		<xsl:call-template name="lookupText">
			<xsl:with-param name="key" select="'localsimpletype'" />
		</xsl:call-template>
	</xsl:variable>

  	<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="$mytitle" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:call-template name="processSimpleType">
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth + 1" />
  		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:call-template>

  </xsl:template>

  <!--
 #####   #####    ####    ####   ######   ####    ####    ####      #    #    # 
 #    #  #    #  #    #  #    #  #       #       #       #          #    ##  ## 
 #    #  #    #  #    #  #       #####    ####    ####    ####      #    # ## # 
 #####   #####   #    #  #       #            #       #       #     #    #    # 
 #       #   #   #    #  #    #  #       #    #  #    #  #    #     #    #    # 
 #       #    #   ####    ####   ######   ####    ####    ####      #    #    # 
-->
  <xsl:template name="processSimpleType">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="depth" select="$depth" />
  		<xsl:with-param name="allancestors" select="$allancestors" />
  	</xsl:apply-templates>
  </xsl:template>

<!--
  ####    ####   #    #  #####   #       ######  #    #   #####   #   #  #####
 #    #  #    #  ##  ##  #    #  #       #        #  #      #      # #   #    #
 #       #    #  # ## #  #    #  #       #####     ##       #       #    #    #
 #       #    #  #    #  #####   #       #         ##       #       #    #####
 #    #  #    #  #    #  #       #       #        #  #      #       #    #
  ####    ####   #    #  #       ######  ######  #    #     #       #    #
-->
  <xsl:template match="xsd:complexType[@name]">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
    <xsl:param name="allancestors"/>
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'CTD'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="@name" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:call-template name="processComplexType">
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth" />
    	<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:call-template>

  </xsl:template>

<!--
  ####    ####   #    #  #####   #       ######  #    #   #####   #   #  #####
 #    #  #    #  ##  ##  #    #  #       #        #  #      #      # #   #    #
 #       #    #  # ## #  #    #  #       #####     ##       #       #    #    #
 #       #    #  #    #  #####   #       #         ##       #       #    #####
 #    #  #    #  #    #  #       #       #        #  #      #       #    #
  ####    ####   #    #  #       ######  ######  #    #     #       #    #
-->
  <xsl:template match="xsd:complexType[@name]" mode="notreeentry">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
    <xsl:param name="allancestors"/>

  	<xsl:call-template name="processComplexType">
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="depth" select="$depth" />
    	<xsl:with-param name="allancestors" select="$allancestors" />
  	</xsl:call-template>
  </xsl:template>

<!--
  ####    ####   #    #  #####   #       ######  #    #   #####   #   #  #####
 #    #  #    #  ##  ##  #    #  #       #        #  #      #      # #   #    #
 #       #    #  # ## #  #    #  #       #####     ##       #       #    #    #
 #       #    #  #    #  #####   #       #         ##       #       #    #####
 #    #  #    #  #    #  #       #       #        #  #      #       #    #
  ####    ####   #    #  #       ######  ######  #    #     #       #    #
-->
  <xsl:template match="xsd:complexType" mode="localcomplextype">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />
	<xsl:variable name="icontype" select="'CTD'" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquename" />
  	</xsl:variable>

  	<xsl:choose>
  		<xsl:when test="xsd:simpleContent">

  			<xsl:variable name="base" select="xsd:simpleContent/xsd:extension/@base" />
  			<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />

  			<xsl:call-template name="writeTreeEntry">
  				<xsl:with-param name="child" select="$mychild" />
  				<xsl:with-param name="parent" select="$myself" />
  				<xsl:with-param name="title" select="concat($base,' +')" />
  				<xsl:with-param name="target" select="$fullpath" />
  				<xsl:with-param name="iconname" select="$icontype" />
  			</xsl:call-template>

  			<xsl:call-template name="processComplexType">
  				<xsl:with-param name="prefix" select="$fullpath" />
  				<xsl:with-param name="depth" select="$depth" />
  				<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  			</xsl:call-template>
  		</xsl:when>

  		<xsl:when test="xsd:complexContent">

      	<xsl:variable name="base">
      		<xsl:choose>
      			<xsl:when test="xsd:complexContent/xsd:extension/@base">
      				<xsl:value-of select="concat(xsd:complexContent/xsd:extension/@base,' +')" />
      			</xsl:when>
      			<xsl:when test="xsd:complexContent/xsd:restriction/@base">
      				<xsl:value-of select="concat(xsd:complexContent/xsd:restriction/@base,' -')" />
      			</xsl:when>
      		</xsl:choose>
      	</xsl:variable>
      	<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />

      	<xsl:call-template name="writeTreeEntry">
      		<xsl:with-param name="child" select="$mychild" />
      		<xsl:with-param name="parent" select="$myself" />
      		<xsl:with-param name="title" select="$base" />
      		<xsl:with-param name="target" select="$fullpath" />
      		<xsl:with-param name="iconname" select="$icontype" />
      	</xsl:call-template>

      	<xsl:for-each select="xsd:complexContent/xsd:extension|xsd:complexContent/xsd:restriction">
      		<xsl:call-template name="processComplexType">
      			<xsl:with-param name="prefix" select="$fullpath" />
      			<xsl:with-param name="depth" select="$depth" />
      			<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
      		</xsl:call-template>
      	</xsl:for-each>

  		</xsl:when>
  		<xsl:otherwise>
  			<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />

  			<xsl:call-template name="writeTreeEntry">
  				<xsl:with-param name="child" select="$mychild" />
  				<xsl:with-param name="parent" select="$myself" />
  				<xsl:with-param name="title" select="$localComplexTypeString" />
  				<xsl:with-param name="target" select="$fullpath" />
  				<xsl:with-param name="iconname" select="$icontype" />
  			</xsl:call-template>

  			<xsl:call-template name="processComplexType">
  				<xsl:with-param name="prefix" select="$fullpath" />
  				<xsl:with-param name="depth" select="$depth" />
  				<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  			</xsl:call-template>
  		</xsl:otherwise>
  	</xsl:choose>
  </xsl:template>

  <!--
 #####   #####    ####    ####   ######   ####    ####    ####    ####   #    # 
 #    #  #    #  #    #  #    #  #       #       #       #    #  #    #  ##  ## 
 #    #  #    #  #    #  #       #####    ####    ####   #       #    #  # ## # 
 #####   #####   #    #  #       #            #       #  #       #    #  #    # 
 #       #   #   #    #  #    #  #       #    #  #    #  #    #  #    #  #    # 
 #       #    #   ####    ####   ######   ####    ####    ####    ####   #    # 
-->  
  <xsl:template name="processComplexType">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
    <xsl:param name="allancestors"/>

    <xsl:for-each select="*">
  	
		<xsl:choose>
			<xsl:when test="local-name()='sequence' or local-name()='choice' or local-name()='all'">
				<xsl:apply-templates select="." mode="notreeentry">
					<xsl:with-param name="prefix" select="$prefix" />
					<xsl:with-param name="depth" select="$depth + 1" />
					<xsl:with-param name="allancestors" select="$allancestors" />
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="local-name()='group'">
				<xsl:apply-templates select="." mode="notreeentry">
					<xsl:with-param name="prefix" select="$prefix" />
					<xsl:with-param name="depth" select="$depth" />
					<xsl:with-param name="allancestors" select="$allancestors" />
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="local-name()='complexContent' or local-name()='simpleContent'">
				<xsl:apply-templates select=".">
					<xsl:with-param name="prefix" select="$prefix" />
					<xsl:with-param name="depth" select="$depth" />
					<xsl:with-param name="allancestors" select="$allancestors" />
				</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select=".">
					<xsl:with-param name="prefix" select="$prefix" />
					<xsl:with-param name="depth" select="$depth + 1" />
					<xsl:with-param name="allancestors" select="$allancestors" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:for-each>
	
  </xsl:template>

<!--
  ####   #####    ####   #    #  #####
 #    #  #    #  #    #  #    #  #    #
 #       #    #  #    #  #    #  #    #
 #  ###  #####   #    #  #    #  #####
 #    #  #   #   #    #  #    #  #
  ####   #    #   ####    ####   #
-->
  <xsl:template match="xsd:group[@name]" mode="followref">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'MGD'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>

    <xsl:call-template name="processGroup">
        <xsl:with-param name="prefix" select="$fullpath"/>
        <xsl:with-param name="depth" select="$depth"/>
    	<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
    </xsl:call-template>
  </xsl:template>
  
<!--
  ####   #####    ####   #    #  #####
 #    #  #    #  #    #  #    #  #    #
 #       #    #  #    #  #    #  #    #
 #  ###  #####   #    #  #    #  #####
 #    #  #   #   #    #  #    #  #
  ####   #    #   ####    ####   #
-->
  <xsl:template match="xsd:group[@name]">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'MGD'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="@name" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

    <xsl:call-template name="processGroup">
    	<xsl:with-param name="prefix" select="$fullpath"/>
	    <xsl:with-param name="depth" select="$depth"/>
    	<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
    </xsl:call-template>
  </xsl:template>
  
<!--
  ####   #####    ####   #    #  #####
 #    #  #    #  #    #  #    #  #    #
 #       #    #  #    #  #    #  #    #
 #  ###  #####   #    #  #    #  #####
 #    #  #   #   #    #  #    #  #
  ####   #    #   ####    ####   #
-->
  <xsl:template match="xsd:group[@ref]" mode="notreeentry">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>
  	
    <xsl:variable name="type">
    	<xsl:call-template name="adjustnameofentity">
    		<xsl:with-param name="entityname" select="@ref" />
    	</xsl:call-template>
    </xsl:variable>
    <!--
    <xsl:variable name="type" select="@ref"/>
    -->
    <xsl:variable name="newprefix" select="concat($mainframe,'#groups/group/')"/>
    
    <xsl:apply-templates select="/xsd:schema/xsd:group[@name=$type]" mode="followref">
        <xsl:with-param name="prefix" select="$newprefix"/>
        <xsl:with-param name="depth" select="$depth"/>
    	<xsl:with-param name="allancestors" select="$allancestors" />
    </xsl:apply-templates>

  </xsl:template>
  
<!--
  ####   #####    ####   #    #  #####
 #    #  #    #  #    #  #    #  #    #
 #       #    #  #    #  #    #  #    #
 #  ###  #####   #    #  #    #  #####
 #    #  #   #   #    #  #    #  #
  ####   #    #   ####    ####   #
-->
  <xsl:template match="xsd:group[@ref]">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>

    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniqueref" />
    </xsl:variable>

    <xsl:variable name="icontype" select="'MGU'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>
	<xsl:variable name="isRecursive" select="contains($allancestors,$uniqueid)"/>

    <xsl:variable name="title">
    	<xsl:choose>
    		<xsl:when test="$isRecursive">
    			<xsl:value-of select="concat(@ref,$recursionIndicatorString)" />
    		</xsl:when>
    		<xsl:otherwise>
    			<xsl:value-of select="@ref" />
    		</xsl:otherwise>
    	</xsl:choose>
    </xsl:variable>

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="$title" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:if test="not($isRecursive)">
	    <xsl:variable name="type">
    		<xsl:call-template name="adjustnameofentity">
    			<xsl:with-param name="entityname" select="@ref" />
	    	</xsl:call-template>
    	</xsl:variable>
    	<!--
  		<xsl:variable name="type" select="@ref" />
  		-->
  		<xsl:variable name="newfullpath" select="concat($mainframe,'#groups/group/')" />
  		<xsl:apply-templates select="/xsd:schema/xsd:group[@name=$type]" mode="followref">
  			<xsl:with-param name="prefix" select="$newfullpath" />
  			<xsl:with-param name="depth" select="$depth" />
  			<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  		</xsl:apply-templates>
  	</xsl:if>
  </xsl:template>

<!--
 #####   #####    ####    ####   ######   ####    ####    ####   #####    ####  
 #    #  #    #  #    #  #    #  #       #       #       #    #  #    #  #    # 
 #    #  #    #  #    #  #       #####    ####    ####   #       #    #  #    # 
 #####   #####   #    #  #       #            #       #  #  ###  #####   #    # 
 #       #   #   #    #  #    #  #       #    #  #    #  #    #  #   #   #    # 
 #       #    #   ####    ####   ######   ####    ####    ####   #    #   ####  

  	xsd:group must be one of
  		xsd:sequence
  		xsd:choice
  		xsd:all
  	Each of these may contain
  		Local Element
	  	Element Reference
 	 		Wildcard Element
  		Local Group
	  	Group Reference


  		Skip the first level
  		There should only be one item in this loop
  		choice, sequence or any
-->  
  <xsl:template name="processGroup">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
    <xsl:param name="allancestors"/>

  	<xsl:for-each select="*">
  		<xsl:for-each select="*">
  			<xsl:apply-templates select=".">
  				<xsl:with-param name="prefix" select="$prefix" />
  				<xsl:with-param name="depth" select="$depth + 1" />
		    	<xsl:with-param name="allancestors" select="$allancestors" />
  			</xsl:apply-templates>
  		</xsl:for-each>
  	</xsl:for-each>
  </xsl:template>

  <!--
   ##     #####   #####  #####      #    #####   #    #   #####  ######   ####
  #  #      #       #    #    #     #    #    #  #    #     #    #       #    #
 #    #     #       #    #    #     #    #####   #    #     #    #####   #
 ######     #       #    #####      #    #    #  #    #     #    #       #  ###
 #    #     #       #    #   #      #    #    #  #    #     #    #       #    #
 #    #     #       #    #    #     #    #####    ####      #    ######   ####

	  xsd:attributeGroup may contain
	  	Local Attribute
	  	Attribute Reference
	  	Wildcard Attribute
	  	Attribute Group Reference
-->
  <xsl:template match="xsd:attributeGroup[@name]">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>

    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquename" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'AGD'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="@name" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:call-template name="processAttributeGroup">
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth" />
  		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:call-template>
  </xsl:template>

<!--
   ##     #####   #####  #####      #    #####   #    #   #####  ######   ####
  #  #      #       #    #    #     #    #    #  #    #     #    #       #    #
 #    #     #       #    #    #     #    #####   #    #     #    #####   #
 ######     #       #    #####      #    #    #  #    #     #    #       #  ###
 #    #     #       #    #   #      #    #    #  #    #     #    #       #    #
 #    #     #       #    #    #     #    #####    ####      #    ######   ####
-->
  <xsl:template match="xsd:attributeGroup[@name]" mode="followref">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:variable name="uniqueid">
  		<xsl:call-template name="uniquename" />
  	</xsl:variable>

  	<xsl:variable name="icontype" select="'AGD'" />
  	<xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />

  	<xsl:call-template name="processAttributeGroup">
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth" />
  		<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:call-template>
  </xsl:template>

  <!--
   ##     #####   #####  #####      #    #####   #    #   #####  ######   ####
  #  #      #       #    #    #     #    #    #  #    #     #    #       #    #
 #    #     #       #    #    #     #    #####   #    #     #    #####   #
 ######     #       #    #####      #    #    #  #    #     #    #       #  ###
 #    #     #       #    #   #      #    #    #  #    #     #    #       #    #
 #    #     #       #    #    #     #    #####    ####      #    ######   ####
-->
  <xsl:template match="xsd:attributeGroup[@ref]">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
  	<xsl:param name="allancestors" />

    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniqueref" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'AGU'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
    <xsl:variable name="myself" select="concat('aux',$depth+1)"/>
    <xsl:variable name="mychild" select="concat('aux',$depth+2)"/>
    <xsl:variable name="type">
   		<xsl:call-template name="adjustnameofentity">
   			<xsl:with-param name="entityname" select="@ref" />
    	</xsl:call-template>
   	</xsl:variable>
    <xsl:variable name="newfullpath" select="concat($mainframe,'#groups/attributegroup/')"/>
	<xsl:variable name="isRecursive" select="contains($allancestors,$uniqueid)"/>

    <xsl:variable name="title">
    	<xsl:choose>
    		<xsl:when test="$isRecursive">
    			<xsl:value-of select="concat(@ref,$recursionIndicatorString)" />
    		</xsl:when>
    		<xsl:otherwise>
    			<xsl:value-of select="@ref" />
    		</xsl:otherwise>
    	</xsl:choose>
    </xsl:variable>

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="title" select="$title" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  	</xsl:call-template>

  	<xsl:if test="not($isRecursive)">
  		<xsl:apply-templates select="/xsd:schema/xsd:attributeGroup[@name=$type]" mode="followref">
  			<xsl:with-param name="prefix" select="$newfullpath" />
  			<xsl:with-param name="depth" select="$depth" />
  			<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  		</xsl:apply-templates>
  	</xsl:if>

  </xsl:template>

<!--
 #####   #####    ####    ####   ######   ####    ####     ##     #####   ##### 
 #    #  #    #  #    #  #    #  #       #       #        #  #      #       #   
 #    #  #    #  #    #  #       #####    ####    ####   #    #     #       #   
 #####   #####   #    #  #       #            #       #  ######     #       #   
 #       #   #   #    #  #    #  #       #    #  #    #  #    #     #       #   
 #       #    #   ####    ####   ######   ####    ####   #    #     #       #   
-->
  <xsl:template name="processAttributeGroup">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />
  	<xsl:variable name="mygrandchild" select="concat('aux',$depth+3)" />

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="depth" select="$depth + 1" />
  		<xsl:with-param name="allancestors" select="$allancestors" />
  	</xsl:apply-templates>

  </xsl:template>

  <!--
  ####   ######   ####   #    #  ######  #    #   ####   ###### 
 #       #       #    #  #    #  #       ##   #  #    #  #      
  ####   #####   #    #  #    #  #####   # #  #  #       #####  
      #  #       #  # #  #    #  #       #  # #  #       #      
 #    #  #       #   #   #    #  #       #   ##  #    #  #      
  ####   ######   ### #   ####   ######  #    #   ####   ###### 
-->
  <xsl:template match="xsd:sequence[xsd:annotation/xsd:appinfo/MRLocalGroup/@composition='orderedSet']">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
    <xsl:param name="allancestors"/>
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquewildcard" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'OS'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  		<xsl:with-param name="title" select="$orderedSetString" />
  	</xsl:call-template>

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth + 1" />
    	<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:apply-templates>
  </xsl:template>    

<!--
  ####   ######   ####   #    #  ######  #    #   ####   ###### 
 #       #       #    #  #    #  #       ##   #  #    #  #      
  ####   #####   #    #  #    #  #####   # #  #  #       #####  
      #  #       #  # #  #    #  #       #  # #  #       #      
 #    #  #       #   #   #    #  #       #   ##  #    #  #      
  ####   ######   ### #   ####   ######  #    #   ####   ###### 
-->
  <xsl:template match="xsd:sequence[xsd:annotation/xsd:appinfo/MRLocalGroup/@composition='unorderedSet']">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
    <xsl:param name="allancestors"/>
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquewildcard" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'US'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  		<xsl:with-param name="title" select="$unorderedSetString" />
  	</xsl:call-template>

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth + 1" />
    	<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:apply-templates>
  </xsl:template>    

<!--
 ######  #       ######  #    #  ######  #    #   #####
 #       #       #       ##  ##  #       ##   #     #
 #####   #       #####   # ## #  #####   # #  #     #
 #       #       #       #    #  #       #  # #     #
 #       #       #       #    #  #       #   ##     #
 ######  ######  ######  #    #  ######  #    #     #
-->
  <xsl:template match="xsd:element" mode="embeddedsimpletype">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
    <xsl:param name="allancestors"/>
	
	<!--  	
    <xsl:call-template name="blockcomment">
        <xsl:with-param name="text">
            <xsl:text>Embedded Simple Type</xsl:text>
        </xsl:with-param>
    </xsl:call-template>
	-->

    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquewildcard" />
    </xsl:variable>
    
    <xsl:variable name="title">
    	<xsl:value-of select="@type"/>
    </xsl:variable>

	<xsl:variable name="icontype" select="'EST'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  		<xsl:with-param name="title" select="$title" />
  	</xsl:call-template>

  </xsl:template>    

<!--
 #####   ######   ####    #####  #####      #     ####    #####     #     ####  
 #    #  #       #          #    #    #     #    #    #     #       #    #    # 
 #    #  #####    ####      #    #    #     #    #          #       #    #    # 
 #####   #            #     #    #####      #    #          #       #    #    # 
 #   #   #       #    #     #    #   #      #    #    #     #       #    #    # 
 #    #  ######   ####      #    #    #     #     ####      #       #     ####  
-->  
  <xsl:template match="xsd:restriction|xsd:extension">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />

    <xsl:variable name="isglobaltype">
        <xsl:call-template name="lookupglobaltype">
            <xsl:with-param name="type" select="@base"/>
        </xsl:call-template>
    </xsl:variable>

    <xsl:if test="$isglobaltype='True'">
        <xsl:if test="local-name(..)='simpleType'">
            <xsl:variable name="type" select="@base"/>
            <xsl:for-each select="/xsd:schema/xsd:simpleType[contains($type,@name)]">
                <xsl:variable name="fullpath" select="concat($mainframe,'#types/type/',@name,$pathsep)" />
                <xsl:call-template name="processSimpleType">
                    <xsl:with-param name="prefix" select="$fullpath" />
                    <xsl:with-param name="depth" select="$depth" />
                    <xsl:with-param name="allancestors" select="$allancestors" />
                </xsl:call-template>
            </xsl:for-each>
        </xsl:if>
    </xsl:if>

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="depth" select="$depth" />
    	<xsl:with-param name="allancestors" select="$allancestors" />
  	</xsl:apply-templates>
  </xsl:template>

<!--
 #####   #####    ####    ####   ######   ####    ####   #    #  ######  #    #
 #    #  #    #  #    #  #    #  #       #       #       ##  ##  #       ##  ##
 #    #  #    #  #    #  #       #####    ####    ####   # ## #  #####   # ## #
 #####   #####   #    #  #       #            #       #  #    #  #       #    #
 #       #   #   #    #  #    #  #       #    #  #    #  #    #  #       #    #
 #       #    #   ####    ####   ######   ####    ####   #    #  ######  #    #
-->
   <xsl:template name="processMemberTypes">
       <xsl:param name="prefix" />
       <xsl:param name="depth" />
       <xsl:param name="allancestors" />
       <xsl:param name="types" /> <!-- A space separated list of types -->

       <xsl:variable name="myself" select="concat('aux',$depth+1)" />
       <xsl:variable name="mychild" select="concat('aux',$depth+2)" />

       <xsl:variable name="icontype" select="'STDU'" />

       <xsl:variable name="sb" select="substring-before($types,' ')" />

       <!-- Extract the first type from the list -->
       <xsl:variable name="type">
           <xsl:choose>
               <xsl:when test="$sb">
                   <xsl:value-of select="$sb" />
               </xsl:when>
               <xsl:otherwise>
                   <xsl:value-of select="$types" />
               </xsl:otherwise>
           </xsl:choose>
       </xsl:variable>

       <xsl:variable name="uniqueid">
           <xsl:call-template name="uniquemembertypeid">
               <xsl:with-param name="membertype" select="$type"/>
           </xsl:call-template>
       </xsl:variable>

       <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)" />

       <xsl:call-template name="writeTreeEntry">
           <xsl:with-param name="child" select="$mychild" />
           <xsl:with-param name="parent" select="$myself" />
           <xsl:with-param name="title" select="$type" />
           <xsl:with-param name="target" select="$fullpath" />
           <xsl:with-param name="iconname" select="$icontype" />
       </xsl:call-template>

<!--
          <xsl:for-each select="/xsd:schema/xsd:simpleType">
              <xsl:call-template name="blockcomment">
                  <xsl:with-param name="text">
                      <xsl:text>name(@name) </xsl:text>
                      <xsl:value-of select="name(@name)"/>
                      <xsl:text> local-name(@name) </xsl:text>
                      <xsl:value-of select="local-name(@name)"/>
                      <xsl:text> @name </xsl:text>
                      <xsl:value-of select="@name"/>
                      <xsl:text> $type </xsl:text>
                      <xsl:value-of select="$type"/>
                  </xsl:with-param>
              </xsl:call-template>
          </xsl:for-each>
  -->

       <xsl:for-each select="/xsd:schema/xsd:simpleType[contains($type,@name)]">
           <xsl:variable name="newfullpath" select="concat($mainframe,'#types/type/',@name,$pathsep)" />
           <xsl:call-template name="processSimpleType">
               <xsl:with-param name="prefix" select="$newfullpath" />
               <xsl:with-param name="depth" select="$depth+1" />
               <xsl:with-param name="allancestors" select="$allancestors" />
           </xsl:call-template>
<!--  
         <xsl:apply-templates select="." mode="localsimpletype">
             <xsl:with-param name="prefix" select="$prefix"/>
             <xsl:with-param name="depth" select="$depth+1"/>
             <xsl:with-param name="allancestors" select="$allancestors" />
           </xsl:apply-templates>
-->           
       </xsl:for-each>

       <!-- Recurse along to the next type in the list -->
       <xsl:if test="$sb">
           <xsl:call-template name="processMemberTypes">
               <xsl:with-param name="prefix" select="$prefix" />
               <xsl:with-param name="depth" select="$depth" />
               <xsl:with-param name="allancestors" select="$allancestors" />
               <xsl:with-param name="types" select="substring-after($types,' ')" />
           </xsl:call-template>
       </xsl:if>

       </xsl:template>

<!--
 #    #  ######  #    #  #####   ######  #####    #####   #   #  #####   ######
 ##  ##  #       ##  ##  #    #  #       #    #     #      # #   #    #  #
 # ## #  #####   # ## #  #####   #####   #    #     #       #    #    #  #####
 #    #  #       #    #  #    #  #       #####      #       #    #####   #
 #    #  #       #    #  #    #  #       #   #      #       #    #       #
 #    #  ######  #    #  #####   ######  #    #     #       #    #       ######
-->
  <xsl:template match="@memberTypes">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />
  	
 	<xsl:call-template name="processMemberTypes">
		<xsl:with-param name="prefix" select="$prefix" />
  		<xsl:with-param name="depth" select="$depth" />
  		<xsl:with-param name="allancestors" select="$allancestors" />
  		<xsl:with-param name="types" select="." />
	</xsl:call-template>
	
  </xsl:template>

<!--
 #    #  #    #     #     ####   #    #
 #    #  ##   #     #    #    #  ##   #
 #    #  # #  #     #    #    #  # #  #
 #    #  #  # #     #    #    #  #  # #
 #    #  #   ##     #    #    #  #   ##
  ####   #    #     #     ####   #    #
-->
  <xsl:template match="xsd:union">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />
  	
  	<!--
  	Can have either
  	- one memberTypes attribute
  	- multiple local simpleType definitions
  	-->

	<xsl:apply-templates select="@memberTypes">
		<xsl:with-param name="prefix" select="$prefix" />
   		<xsl:with-param name="depth" select="$depth" />
  		<xsl:with-param name="allancestors" select="$allancestors" />
	</xsl:apply-templates>

	<xsl:for-each select="xsd:simpleType">
		<xsl:choose>

		  <xsl:when test="xsd:union">
      		<xsl:apply-templates select="." mode="localsimpletypeunion">
	  			<xsl:with-param name="prefix" select="$prefix" />
	  			<xsl:with-param name="depth" select="$depth" />
    			<xsl:with-param name="allancestors" select="$allancestors" />
      		</xsl:apply-templates>
		  </xsl:when>

		  <xsl:otherwise>
      		<xsl:apply-templates select="." mode="localsimpletype">
      			<xsl:with-param name="prefix" select="$prefix" />
	      		<xsl:with-param name="depth" select="$depth" />
    	  		<xsl:with-param name="allancestors" select="$allancestors" />
      		</xsl:apply-templates>
		  </xsl:otherwise>

		</xsl:choose>
	</xsl:for-each>
  </xsl:template>

<!--
 #          #     ####    #####
 #          #    #          #
 #          #     ####      #
 #          #         #     #
 #          #    #    #     #
 ######     #     ####      #
-->
  <xsl:template match="xsd:list">
      <xsl:param name="prefix" />
      <xsl:param name="depth" />
      <xsl:param name="allancestors" />

      <xsl:apply-templates select="xsd:simpleType" mode="localsimpletype">
          <xsl:with-param name="prefix" select="$prefix" />
          <xsl:with-param name="depth" select="$depth" />
          <xsl:with-param name="allancestors" select="$allancestors" />
      </xsl:apply-templates>
      
  </xsl:template>

<!--
  ####    ####   #    #  #####   #       ######  #    #   ####    ####   #    #
 #    #  #    #  ##  ##  #    #  #       #        #  #   #    #  #    #  ##   #
 #       #    #  # ## #  #    #  #       #####     ##    #       #    #  # #  #
 #       #    #  #    #  #####   #       #         ##    #       #    #  #  # #
 #    #  #    #  #    #  #       #       #        #  #   #    #  #    #  #   ##
  ####    ####   #    #  #       ######  ######  #    #   ####    ####   #    #
-->
  <xsl:template match="xsd:complexContent">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />
  	
    <xsl:for-each select="xsd:extension">
        <xsl:variable name="isglobaltype">
            <xsl:call-template name="lookupglobaltype">
                <xsl:with-param name="type" select="@base"/>
            </xsl:call-template>
        </xsl:variable>

        <xsl:if test="$isglobaltype='True'">
            <xsl:variable name="myself" select="concat('aux',$depth+2)" />
            <xsl:variable name="mychild" select="concat('aux',$depth+3)" />

            <xsl:variable name="uniqueid">
                <xsl:call-template name="uniquename" />
            </xsl:variable>

            <xsl:variable name="icontype" select="'BCT'" />
            <xsl:variable name="type" select="@base" />
            <xsl:variable name="fullpath" select="concat($mainframe,'#types/type/',$type,$pathsep)" />

            <xsl:call-template name="writeTreeEntry">
                <xsl:with-param name="child" select="$mychild" />
                <xsl:with-param name="parent" select="$myself" />
                <xsl:with-param name="title" select="concat('{ ',$type,' }')" />
                <xsl:with-param name="target" select="$fullpath" />
                <xsl:with-param name="iconname" select="$icontype" />
            </xsl:call-template>

            <xsl:for-each select="/xsd:schema/xsd:complexType[@name=$type]">
                <xsl:call-template name="processComplexType">
                    <xsl:with-param name="prefix" select="$fullpath" />
                    <xsl:with-param name="depth" select="$depth+1" />
                    <xsl:with-param name="allancestors" select="$allancestors" />
                </xsl:call-template>
            </xsl:for-each>
        </xsl:if>
    </xsl:for-each>

    <xsl:for-each select="xsd:extension|xsd:restriction">
  		<xsl:call-template name="processComplexType">
  			<xsl:with-param name="prefix" select="$prefix" />
  			<xsl:with-param name="depth" select="$depth" />
  			<xsl:with-param name="allancestors" select="$allancestors" />
  		</xsl:call-template>
  	</xsl:for-each>

  </xsl:template>

  <!--
  ####      #    #    #  #####   #       ######   ####    ####   #    #   #####
 #          #    ##  ##  #    #  #       #       #    #  #    #  ##   #     #
  ####      #    # ## #  #    #  #       #####   #       #    #  # #  #     #
      #     #    #    #  #####   #       #       #       #    #  #  # #     #
 #    #     #    #    #  #       #       #       #    #  #    #  #   ##     #
  ####      #    #    #  #       ######  ######   ####    ####   #    #     #
-->
  <xsl:template match="xsd:simpleContent">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />
  	
    <xsl:for-each select="xsd:extension|xsd:restriction">
  		<xsl:call-template name="processComplexType">
  			<xsl:with-param name="prefix" select="$prefix" />
  			<xsl:with-param name="depth" select="$depth" />
  			<xsl:with-param name="allancestors" select="$allancestors" />
  		</xsl:call-template>
  	</xsl:for-each>

  </xsl:template>

  <!--
  ####   ######   ####   #    #  ######  #    #   ####   ###### 
 #       #       #    #  #    #  #       ##   #  #    #  #      
  ####   #####   #    #  #    #  #####   # #  #  #       #####  
      #  #       #  # #  #    #  #       #  # #  #       #      
 #    #  #       #   #   #    #  #       #   ##  #    #  #      
  ####   ######   ### #   ####   ######  #    #   ####   ###### 
-->
  <xsl:template match="xsd:sequence">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
    <xsl:param name="allancestors"/>
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquewildcard" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'MGS'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />

  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  		<xsl:with-param name="title" select="$sequenceString" />
  	</xsl:call-template>

  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth + 1" />
    	<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:apply-templates>
  </xsl:template>

<!--
  ####   #    #   ####      #     ####   ###### 
 #    #  #    #  #    #     #    #    #  #      
 #       ######  #    #     #    #       #####  
 #       #    #  #    #     #    #       #      
 #    #  #    #  #    #     #    #    #  #      
  ####   #    #   ####      #     ####   ###### 
-->
  <xsl:template match="xsd:choice">
    <xsl:param name="prefix"/>
    <xsl:param name="depth"/>
    <xsl:param name="allancestors"/>
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquewildcard" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'MGC'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />
  	
  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  		<xsl:with-param name="title" select="$choiceString" />
  	</xsl:call-template>
  	
  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth + 1" />
    	<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:apply-templates>
  </xsl:template>

<!--
   ##    #       #       
  #  #   #       #       
 #    #  #       #       
 ######  #       #       
 #    #  #       #       
 #    #  ######  ######  
-->
  <xsl:template match="xsd:all">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
    <xsl:param name="allancestors"/>
  	
    <xsl:variable name="uniqueid">
    	<xsl:call-template name="uniquewildcard" />
    </xsl:variable>

	<xsl:variable name="icontype" select="'MGA'" />
    <xsl:variable name="fullpath" select="concat($prefix,$uniqueid,$pathsep)"/>
  	<xsl:variable name="myself" select="concat('aux',$depth+1)" />
  	<xsl:variable name="mychild" select="concat('aux',$depth+2)" />
  	
  	<xsl:call-template name="writeTreeEntry">
  		<xsl:with-param name="child" select="$mychild" />
  		<xsl:with-param name="parent" select="$myself" />
  		<xsl:with-param name="target" select="$fullpath" />
  		<xsl:with-param name="iconname" select="$icontype" />
  		<xsl:with-param name="title" select="$allString" />
  	</xsl:call-template>
  	
  	<xsl:apply-templates>
  		<xsl:with-param name="prefix" select="$fullpath" />
  		<xsl:with-param name="depth" select="$depth + 1" />
    	<xsl:with-param name="allancestors" select="concat($allancestors,'(',$uniqueid,')')" />
  	</xsl:apply-templates>
  </xsl:template>

<!--
  ####   ######   ####     #      ####   #    #   ####     #       ##    #      
 #       #       #    #    #     #    #  #    #  #    #    #      #  #   #      
  ####   #####   #    #  #####   #       ######  #    #  #####   #    #  #      
      #  #       #  # #    #     #       #    #  #    #    #     ######  #      
 #    #  #       #   #     #     #    #  #    #  #    #    #     #    #  #      
  ####   ######   ### #           ####   #    #   ####           #    #  ###### 
-->
  <xsl:template match="xsd:sequence|xsd:choice|xsd:all" mode="notreeentry">
  	<xsl:param name="prefix" />
  	<xsl:param name="depth" />
  	<xsl:param name="allancestors" />
    <xsl:for-each select="*">
        <xsl:choose>
            <!-- 
                Embedded simple types have specific rules.
                Modify this line as the rules alter
                Removed from test "../@mixed='true' and "
            <xsl:when test="local-name()='sequence' and xsd:element/@maxOccurs='0' and xsd:element/@minOccurs='0' and starts-with(xsd:element/@type,'ComIbmMrm_Anon')">
            -->
            <xsl:when test="local-name(..)='sequence' and local-name()='element' and @maxOccurs='0' and @minOccurs='0' and starts-with(@type,'ComIbmMrm_Anon')">
                <xsl:apply-templates select="." mode="embeddedsimpletype">
                    <xsl:with-param name="prefix" select="$prefix" />
                    <xsl:with-param name="depth" select="$depth" />
                    <xsl:with-param name="allancestors" select="$allancestors" />
                </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select=".">
                    <xsl:with-param name="prefix" select="$prefix" />
                    <xsl:with-param name="depth" select="$depth" />
                    <xsl:with-param name="allancestors" select="$allancestors" />
                </xsl:apply-templates>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:for-each>
  </xsl:template>

  <!--
 #####    ####    ####   #    #  #    #  ######  #    #   #####    ##     ##### 
 #    #  #    #  #    #  #    #  ##  ##  #       ##   #     #     #  #      #   
 #    #  #    #  #       #    #  # ## #  #####   # #  #     #    #    #     #   
 #    #  #    #  #       #    #  #    #  #       #  # #     #    ######     #   
 #    #  #    #  #    #  #    #  #    #  #       #   ##     #    #    #     #   
 #####    ####    ####    ####   #    #  ######  #    #     #    #    #     #   
-->
  <xsl:template match="xsd:documentation" />
  
<!--
   ##    #    #  #    #   ####    #####    ##     #####     #     ####   #    # 
  #  #   ##   #  ##   #  #    #     #     #  #      #       #    #    #  ##   # 
 #    #  # #  #  # #  #  #    #     #    #    #     #       #    #    #  # #  # 
 ######  #  # #  #  # #  #    #     #    ######     #       #    #    #  #  # # 
 #    #  #   ##  #   ##  #    #     #    #    #     #       #    #    #  #   ## 
 #    #  #    #  #    #   ####      #    #    #     #       #     ####   #    # 
-->  
  <xsl:template match="xsd:annotation" />

<!--
 #    #  #####      #     #####  ######   #####  #####   ######  ######  ###### 
 #    #  #    #     #       #    #          #    #    #  #       #       #      
 #    #  #    #     #       #    #####      #    #    #  #####   #####   #####  
 # ## #  #####      #       #    #          #    #####   #       #       #      
 ##  ##  #   #      #       #    #          #    #   #   #       #       #      
 #    #  #    #     #       #    ######     #    #    #  ######  ######  ###### 
-->
  <xsl:template name="writeTreeEntry">
  	<xsl:param name="child" select="'aux1'" />
  	<xsl:param name="parent" select="'aux0'" />
  	<xsl:param name="title" select="'No Title'" />
  	<xsl:param name="target" select="'No Target'"/>
  	<xsl:param name="iconname" select="'ZZ'" />

  	<xsl:value-of select="$child" />
  	<xsl:text>=insFld(</xsl:text>
  	<xsl:value-of select="$parent" />
  	<xsl:text>,gFld("</xsl:text>
  	<xsl:value-of select="$title" />
  	<xsl:text>","</xsl:text>
  	<xsl:value-of disable-output-escaping="yes" select="$target" />
  	<xsl:text>","</xsl:text>
  	<xsl:value-of select="$iconname" />
  	<xsl:text>"))&eol;</xsl:text>

  </xsl:template>

<!--
 #####   #        ####    ####   #    #   ####    ####   #    #  #    #  ###### 
 #    #  #       #    #  #    #  #   #   #    #  #    #  ##  ##  ##  ##  #      
 #####   #       #    #  #       ####    #       #    #  # ## #  # ## #  #####  
 #    #  #       #    #  #       #  #    #       #    #  #    #  #    #  #      
 #    #  #       #    #  #    #  #   #   #    #  #    #  #    #  #    #  #      
 #####   ######   ####    ####   #    #   ####    ####   #    #  #    #  ###### 
-->
  <xsl:template name="blockcomment">
    <xsl:param name="text">No comment</xsl:param>
   	<xsl:text>// =======================&eol;</xsl:text>
   	<xsl:text>// </xsl:text>
   	<xsl:value-of select="$text" />
   	<xsl:text>&eol;</xsl:text>
   	<xsl:text>// =======================&eol;</xsl:text>
  </xsl:template>

<!--
 #    #  #    #     #     ####   #    #  ######  #    #    ##    #    #  ###### 
 #    #  ##   #     #    #    #  #    #  #       ##   #   #  #   ##  ##  #      
 #    #  # #  #     #    #    #  #    #  #####   # #  #  #    #  # ## #  #####  
 #    #  #  # #     #    #  # #  #    #  #       #  # #  ######  #    #  #      
 #    #  #   ##     #    #   #   #    #  #       #   ##  #    #  #    #  #      
  ####   #    #     #     ### #   ####   ######  #    #  #    #  #    #  ###### 
-->
    <xsl:template name="uniquename">
    	<xsl:variable name="nances" select="count(ancestor::*)" />
    	<xsl:if test="$nances=1">
    		<xsl:value-of select="@name" />
    	</xsl:if>
    	<xsl:if test="$nances!=1">
    		<xsl:value-of select="generate-id()" />
    	</xsl:if>
    </xsl:template>

    <xsl:template name="uniquemembertypeid">
        <xsl:param name="membertype"/>
        <xsl:value-of select="concat(generate-id(),'.',$membertype)" />
    </xsl:template>

<!--
 #    #  #    #     #     ####   #    #  ######  #####   ######  ######
 #    #  ##   #     #    #    #  #    #  #       #    #  #       #
 #    #  # #  #     #    #    #  #    #  #####   #    #  #####   #####
 #    #  #  # #     #    #  # #  #    #  #       #####   #       #
 #    #  #   ##     #    #   #   #    #  #       #   #   #       #
  ####   #    #     #     ### #   ####   ######  #    #  ######  #
-->
    <xsl:template name="uniqueref">
    	<xsl:variable name="nances" select="count(ancestor::*)" />
    	<xsl:if test="$nances=1">
    		<xsl:variable name="atname" select="@ref" />
    		<xsl:variable name="myname" select="name()" />
    		<xsl:variable name="count" select="count(preceding-sibling::*[@ref=$atname and name()=$myname])" />
    		<xsl:value-of select="concat($atname,'(',$count,')')" />
    	</xsl:if>
    	<xsl:if test="$nances!=1">
    		<xsl:value-of select="generate-id()" />
    	</xsl:if>
    </xsl:template>

<!--
 #    #  #    #     #     ####   #    #  ######  #    #     #    #       #####
 #    #  ##   #     #    #    #  #    #  #       #    #     #    #       #    #
 #    #  # #  #     #    #    #  #    #  #####   #    #     #    #       #    #
 #    #  #  # #     #    #  # #  #    #  #       # ## #     #    #       #    #
 #    #  #   ##     #    #   #   #    #  #       ##  ##     #    #       #    #
  ####   #    #     #     ### #   ####   ######  #    #     #    ######  #####
-->
    <xsl:template name="uniquewildcard">
    	<xsl:variable name="nances" select="count(ancestor::*)" />
    	<xsl:if test="$nances=1">
    		<xsl:variable name="myname" select="name()" />
    		<xsl:variable name="count" select="count(preceding-sibling::*[name()=$myname])" />
    		<xsl:value-of select="concat(local-name(),'(',$count,')')" />
    	</xsl:if>
    	<xsl:if test="$nances!=1">
    		<xsl:value-of select="generate-id()" />
    	</xsl:if>
    </xsl:template>

     <xsl:key name="globalentities" match="kvpair" use="@key"/>
     <xsl:variable name="globalentityfile" select="concat($outputPath,'/globals.lut.xml')"/>

     <xsl:template name="lookupglobal">
       <xsl:param name="key" select="NoKeySupplied" />
       <xsl:param name="text" select="NoTextSupplied" />
       <xsl:for-each select="document($globalentityfile)/toc">
           <xsl:choose>
               <xsl:when test="key('lookupText',$key)/@value">
                   <xsl:text>True</xsl:text>
               </xsl:when>
               <xsl:otherwise>
                   <xsl:text>False</xsl:text>
               </xsl:otherwise>
           </xsl:choose>
       </xsl:for-each>
     </xsl:template>

     <xsl:template name="expandNamespacePrefix">
         <xsl:param name="namespaceprefix" select="NoNamespacePrefixSupplied"/>
         <xsl:for-each select="/xsd:schema/namespace::*">
             <xsl:if test="local-name() = $namespaceprefix">
                 <xsl:value-of select="self::node()"/>
             </xsl:if>
         </xsl:for-each>
     </xsl:template>

     <xsl:template name="lookupglobalprologue">
       <xsl:param name="entitytype" select="NoEntityTypeSupplied"/>
       <xsl:param name="entityname" select="NoEntityNameSupplied"/>

       <xsl:variable name="key">
           <xsl:choose>
               <xsl:when test="contains($entityname,':')">
                   <xsl:variable name="ns">
                       <xsl:call-template name="expandNamespacePrefix">
                           <xsl:with-param name="namespaceprefix" select="substring-before($entityname,':')"/>
                       </xsl:call-template>
                   </xsl:variable>
                   <xsl:variable name="nn" select="substring-after($entityname,':')"/>
                   <xsl:value-of select="concat($entitytype,'/',$ns,'#',$nn,$pathsep)"/>
               </xsl:when>
               <xsl:otherwise>
                   <xsl:value-of select="concat($entitytype,'/#',$entityname,$pathsep)"/>
               </xsl:otherwise>
           </xsl:choose>
       </xsl:variable>

       <xsl:call-template name="lookupglobal">
           <xsl:with-param name="key" select="$key" />
           <xsl:with-param name="text" select="$entityname" />
       </xsl:call-template>
     </xsl:template>

     <xsl:template name="lookupglobaltype">
       <xsl:param name="type" select="NoTypeSupplied" />
       <xsl:call-template name="lookupglobalprologue">
           <xsl:with-param name="entitytype" select="'types/type'" />
           <xsl:with-param name="entityname" select="$type" />
       </xsl:call-template>
     </xsl:template>

  <!-- The NLS processing is common between the stylesheets -->
  <xsl:include href="docgen_nls.xsl"/>

</xsl:stylesheet>